# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer
from hammr.utils import *
from hammr.utils.hammr_utils import *


class Platform(Cmd, CoreGlobal):
    """List platforms available"""

    cmd_name = "platform"

    def __init__(self):
        super(Platform, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(add_help=True, description="List all the target platform I can access.")
        return doParser

    def do_list(self, args):
        try:
            printer.out("Getting target platform :")
            targetPlatformsUser = self.api.Users(self.login).Targetplatforms.Getall()
            if targetPlatformsUser is None or len(targetPlatformsUser.targetPlatforms.targetPlatform) == 0:
                printer.out("There is no target platform")
                return 0
            else:
                targetPlatformsUser = generics_utils.order_list_object_by(
                    targetPlatformsUser.targetPlatforms.targetPlatform, "name")
                printer.out("Target platform list:")
                table = Texttable(200)
                table.set_cols_align(["c", "c", "c", "c"])
                table.header(["Id", "Name", "Type", "Access"])
                for item in targetPlatformsUser:
                    if item.access:
                        access = "X"
                    else:
                        access = ""
                    table.add_row([item.dbId, item.name, item.type, access])
                print table.draw() + "\n"
            return 0

        except ArgumentParserError as e:
            printer.out("In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()



# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from mock import patch
from uforge.application import Api
from uforge.objects.uforge import *

from hammr.commands.scan import scan
from hammr.commands.scan.scan import Scan
from hammr.utils import constants


class TestScan(TestCase):

    @patch("ussclicore.utils.download_utils.Download")
    @patch.object(Scan, "deploy_and_launch_agent")
    @patch('uforge.application.Api._Users._Scannedinstances.Getall')
    @patch("ussclicore.utils.printer.out")
    def test_do_run_return_error_when_regular_scan_and_overlay_option(self, mock_out, mock_api_getall, mock_agent, mock_download):
        # given
        name = "RegularScan"
        s, args = self.prepare_scan_run_command(name, "--overlay")

        self.create_scanned_instances(name, False, mock_api_getall)

        # when
        s.do_run(args)

        # then
        self.assertEqual(s.check_overlay_option_is_allowed(name, True), False)
        mock_out.assert_called_with("Performing scan with overlay into the scanned instance [RegularScan] is not allowed. Please retry with another one.", "ERROR")
        self.assertEquals(mock_download.call_count, 0)
        self.assertEquals(mock_agent.call_count, 0)


    @patch("ussclicore.utils.download_utils.Download")
    @patch.object(Scan, "deploy_and_launch_agent")
    @patch('uforge.application.Api._Users._Scannedinstances.Getall')
    @patch("ussclicore.utils.printer.out")
    def test_do_run_return_error_when_scan_with_overlay_and_no_overlay_option(self, mock_out, mock_api_getall, mock_agent, mock_download):
        # given
        name = "ScanWithOverlay"
        s, args = self.prepare_scan_run_command(name, "")

        self.create_scanned_instances(name, True, mock_api_getall)

        # when
        s.do_run(args)

        # then
        self.assertEqual(s.check_overlay_option_is_allowed(name, False), False)
        mock_out.assert_called_with(
            "Performing regular scan into the scanned instance [ScanWithOverlay] is not allowed. Please retry with another one.",
            "ERROR")
        self.assertEquals(mock_download.call_count, 0)
        self.assertEquals(mock_agent.call_count, 0)

    @patch("ussclicore.utils.download_utils.Download")
    @patch.object(Scan, "deploy_and_launch_agent")
    @patch('uforge.application.Api._Users._Scannedinstances.Getall')
    def test_do_run_succeed_when_scan_with_overlay_and_overlay_option(self, mock_api_getall, mock_agent, mock_download):
        #given
        name = "ScanWithOverlay"
        s, args = self.prepare_scan_run_command(name, "--overlay")

        self.create_scanned_instances(name, True, mock_api_getall)
        mock_download.start.return_value = ''

        # when
        s.do_run(args)

        # then
        self.assertEqual(s.check_overlay_option_is_allowed(name,True), True)
        self.assertEquals(mock_download.call_count, 1)
        self.assertEquals(mock_agent.call_count, 1)

    @patch("ussclicore.utils.download_utils.Download")
    @patch.object(Scan, "deploy_and_launch_agent")
    @patch('uforge.application.Api._Users._Scannedinstances.Getall')
    def test_do_run_succeed_when_regular_scan_and_no_overlay_option(self, mock_api_getall, mock_agent, mock_download):
        # given
        name = "RegularScan"
        s, args = self.prepare_scan_run_command(name, "")

        self.create_scanned_instances(name, False, mock_api_getall)
        mock_download.start.return_value = ''

        # when
        s.do_run(args)

        # then
        self.assertEqual(s.check_overlay_option_is_allowed(name, False), True)
        self.assertEquals(mock_download.call_count, 1)
        self.assertEquals(mock_agent.call_count, 1)

    def create_scanned_instances(self, name, overlay, mock_api_getall):
        scanned_instances = scannedInstances()
        scanned_instances.scannedInstances = pyxb.BIND()

        newScannedInstance = scannedInstance()
        newScannedInstance.name = name
        newScannedInstance.overlayIncluded = overlay
        scanned_instances.scannedInstances.append(newScannedInstance)

        mock_api_getall.return_value = scanned_instances

    def prepare_scan_run_command(self, name, overlay_option):
        s = scan.Scan()
        s.api = Api("url", username="username", password="password", headers=None,
                    disable_ssl_certificate_validation=False, timeout=constants.HTTP_TIMEOUT)
        s.login = "login"
        s.password = "password"

        args = "--ip 10.0.0.1 --scan-login login --scan-password password %s --name '%s'" % (overlay_option, name)
        return s, args





# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
import pyxb
from uforge.objects.uforge import *
import ussclicore.utils.download_utils
from ussclicore.utils import printer
from ussclicore.utils import generics_utils
from hammr_utils import *
from progressbar import Bar, ProgressBar, ReverseBar, UnknownLength, BouncingBar
from ussclicore.utils import progressbar_widget
import pyxb.binding.content as pyxb_content
from texttable import Texttable
import getpass
import re

def retrieve_credaccount(image_object, pimageId, pimage):
    # Increases the limit for non determinist content: the xml used for openstack retrieval has a lot of
    # non determinist content that raises an exception if limit is low.
    # See https://stackoverflow.com/questions/1952931/how-to-rewrite-this-nondeterministic-xml-schema-to-deterministic
    pyxb_content.AutomatonConfiguration.PermittedNondeterminism = 10000
    if is_uri_based_on_appliance(pimage.uri):
        return retrieve_credaccount_from_app(image_object, pimageId, pimage)
    else:
        return retrieve_credaccount_from_scan(image_object, pimageId, pimage)

def retrieve_credaccount_from_app(image_object, pimageId, pimage):
    image_id = generics_utils.extract_id(pimage.imageUri)
    source_id = generics_utils.extract_id(pimage.applianceUri)
    account_id = pimage.credAccount.dbId
    return image_object.api.Users(image_object.login).Appliances(source_id).Images(image_id).Pimages(pimageId).\
        Accounts(account_id).Resources.Getaccountresources()

def retrieve_credaccount_from_scan(image_object, pimageId, pimage):
    image_id = generics_utils.extract_id(pimage.imageUri)
    scannedinstance_id = extract_scannedinstance_id(pimage.imageUri)
    scan_id = extract_scan_id(pimage.imageUri)
    account_id = pimage.credAccount.dbId
    return image_object.api.Users(image_object.login).Scannedinstances(scannedinstance_id).Scans(scan_id).\
        Images(image_id).Pimages(pimageId).Accounts(account_id).Resources.Getaccountresources()

def validate_deployment(file):
    try:
        isJson = check_extension_is_json(file)
        if isJson:
            data = generics_utils.check_json_syntax(file)
        else:
            data = generics_utils.check_yaml_syntax(file)

        if data is None:
            return
        if "provisioner" not in data:
            printer.out("There is no provisioner in the file", printer.ERROR)
        return data["provisioner"]

    except ValueError as e:
        printer.out("JSON parsing error: " + str(e), printer.ERROR)
        printer.out("Syntax of deployment file [" + file + "]: FAILED")
    except IOError as e:
        printer.out("unknown error deployment json file", printer.ERROR)

def build_deployment_amazon(data):
    data = validate_deployment(data)
    deployment = Deployment()
    myinstance = InstanceAmazon()
    if not "name" in data:
        printer.out("There is no attribute [name] for the provisioner", printer.ERROR)
        return None
    deployment.name = data["name"]
    if not "cores" in data:
        myinstance.cores = "1"
    else:
        myinstance.cores = data["cores"]
    if not "memory" in data:
        myinstance.memory = "1024"
    else:
        myinstance.memory = data["memory"]
    deployment.instances = pyxb.BIND()
    deployment.instances._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Instances')
    deployment.instances.append(myinstance)
    return deployment

def build_deployment_azure(data):
    data = validate_deployment(data)
    deployment = Deployment()
    myinstance = InstanceAzureResourceManager()
    if not "name" in data:
        printer.out("There is no attribute [name] for the provisioner", printer.ERROR)
        return None
    deployment.name = data["name"]
    if "userName" in data:
        myinstance.userName = data["userName"]
    else:
        printer.out("There is no attribute [userName] for the provisioner", printer.ERROR)
        return None
    if "userSshKey" in data:
        myinstance.userSshKey = data["userSshKey"]
    elif "userSshKeyFile" in data:
        try:
            myinstance.userSshKey = open(data["userSshKeyFile"], "r").read()
        except IOError as e:
                printer.out("File error: "+str(e), printer.ERROR)
                return
    else:
        myinstance.userPassword = query_password_azure("Please enter the password to connect to the instance: ")

    if not "cores" in data:
        myinstance.cores = "1"
    else:
        myinstance.cores = data["cores"]
    if not "memory" in data:
        myinstance.memory = "1024"
    else:
        myinstance.memory = data["memory"]

    deployment.instances = pyxb.BIND()
    deployment.instances._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Instances')
    deployment.instances.append(myinstance)
    return deployment

def build_deployment_openstack(data, pimage, pimageId, cred_account_ressources):
    data = validate_deployment(data)
    deployment = Deployment()
    myinstance = InstanceOpenStack()

    if not "name" in data:
        printer.out("There is no attribute [name] for the provisioner", printer.ERROR)
        return None
    deployment.name = data["name"]

    if not "region" in data:
        printer.out("There is no attribute [region] for the provisioner", printer.ERROR)
        return None
    myinstance.region = data["region"]

    if not "network" in data:
        printer.out("There is no attribute [network] for the provisioner", printer.ERROR)
        return None
    network_name = data["network"]

    if not "flavor" in data:
        printer.out("There is no attribute [flavor] for the provisioner", printer.ERROR)
        return None
    flavor_name = data["flavor"]

    myinstance.networkId, myinstance.flavorId = retrieve_openstack_resources(myinstance.region, network_name,
                                                                flavor_name, pimage, pimageId, cred_account_ressources)

    deployment.instances = pyxb.BIND()
    deployment.instances._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Instances')
    deployment.instances.append(myinstance)

    return deployment

def retrieve_openstack_resources(region_name, network_name, flavor_name, pimage, pimageId, cred_account_ressources):
    flavor_id = None
    network_id = None
    tenants = cred_account_ressources.cloudResources.tenants.tenant
    tenant_name = pimage.tenantName
    for tenant in tenants:
        if tenant.name == tenant_name:
            break;

    region_retrieved = None
    regionsEntities = tenant.regionsEntities
    for regionEntities in regionsEntities:
        regions = regionEntities.regionEntities
        for region in regions:
            if region.regionName == region_name:
                region_retrieved = region
                break;

    if region_retrieved == None:
        printer.out("Region of the published image not found", printer.ERROR)
        return None, None

    flavors = region.flavors.flavor
    for flavor in flavors:
        if flavor.name == flavor_name:
            flavor_id = flavor.id
            break;

    networks = region.networks.network
    for network in networks:
        if network.name == network_name:
            network_id = network.id
            break;

    return network_id[0].encode('ascii', 'ignore'), flavor_id.encode('ascii', 'ignore')

def create_progress_bar_openstack(bar_status):
    bar_status.message = "Retrieving information from OpenStack"
    bar_status.percentage = 0
    statusWidget = progressbar_widget.Status()
    statusWidget.status = bar_status
    widgets = [Bar('>'), ' ', statusWidget, ' ', ReverseBar('<')]
    progress = ProgressBar(widgets=widgets, maxval=100).start()
    progress.start()
    bar_status.percentage = 10
    progress.update(bar_status.percentage)
    return progress

def call_deploy(image_object, pimage, deployment, image_id):
    if is_uri_based_on_appliance(pimage.imageUri):
        source = image_object.api.Users(image_object.login).Appliances(generics_utils.extract_id(pimage.applianceUri)).Get()
        if source is None or not hasattr(source, 'dbId'):
            printer.out("No template found for this image", printer.ERROR)
            return 2
        deployed_instance = image_object.api.Users(image_object.login).Appliances(source.dbId).Images(image_id).Pimages(
            pimage.dbId).Deploys.Deploy(body=deployment, element_name="ns1:deployment")
    elif is_uri_based_on_scan(pimage.imageUri):
        ScannedInstanceId = extract_scannedinstance_id(pimage.imageUri)
        ScanId = extract_scan_id(pimage.imageUri)
        source = image_object.api.Users(image_object.login).Scannedinstances(ScannedInstanceId).Scans(ScanId).Get()
        if source is None or not hasattr(source, 'dbId'):
            printer.out("No scan found for this image", printer.ERROR)
            return 2
        deployed_instance = image_object.api.Users(image_object.login).Scannedinstances(ScannedInstanceId).Scans(
            ScanId).Images(Itid=image_id).Pimages(pimage.dbId).Deploys.Deploy(body=deployment, element_name="ns1:deployment")
    else:
        printer.out("No source found for this image", printer.ERROR)
        return 2
    return deployed_instance

def print_deploy_info(image_object, status, deployed_instance_id):
    if status.message == "on-fire":
        printer.out("Deployment failed", printer.ERROR)
        if status.detailedError:
            printer.out(status.detailedErrorMsg, printer.ERROR)
        return 1
    else:
        printer.out("Deployment is successful", printer.OK)

        deployment = image_object.api.Users(image_object.login).Deployments(deployed_instance_id).Get()
        instance = deployment.instances.instance[-1]

        deployment_name = deployment.name
        cloud_provider = format_cloud_provider(instance.cloudProvider)
        location = instance.location.provider
        hostname=instance.hostname
        deployment_status = deployment.state

        if instance.sourceSummary and type(instance.sourceSummary) == ScanSummaryLight:
            source_type = "Scan"
            source_id = str(extract_scannedinstance_id(instance.sourceSummary.uri))
            source_name = instance.sourceSummary.name
        elif instance.sourceSummary and type(instance.sourceSummary) == ApplianceSummary:
            source_type = "Template"
            source_id = str(generics_utils.extract_id(instance.sourceSummary.uri))
            source_name = instance.sourceSummary.name
        else:
            source_type = None
            source_id = None
            source_name = None

        table = print_deploy_info_line(deployment_name, deployed_instance_id, cloud_provider, location, hostname, source_type,
                               source_id, source_name, deployment_status)
        print table.draw() + "\n"
        return 0

def show_deploy_progress_without_percentage(image_object, deployed_instance_id):
    status = image_object.api.Users(image_object.login).Deployments(deployed_instance_id).Status.Getdeploystatus()
    bar = ProgressBar(widgets=[BouncingBar()], maxval=UnknownLength)
    bar.start()
    i = 1
    while not (status.message == "running" or status.message == "on-fire"):
        status = image_object.api.Users(image_object.login).Deployments(deployed_instance_id).Status.Getdeploystatus()
        time.sleep(1)
        bar.update(i)
        i += 2
    bar.finish()
    return status

def show_deploy_progress_with_percentage(image_object, deployed_instance_id, bar_status, progress):
    status = image_object.api.Users(image_object.login).Deployments(deployed_instance_id).Status.Getdeploystatus()
    while not (status.message == "running" or status.message == "on-fire"):
        status = image_object.api.Users(image_object.login).Deployments(deployed_instance_id).Status.Getdeploystatus()
        time.sleep(1)

    bar_status.percentage = 100
    bar_status.message = "Deployment complete"
    progress.update(bar_status.percentage)
    progress.finish()
    return status

def query_password_azure(question):
    pattern = "(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%^&+=*!])(?=\\S+$).{6,}"
    while(True):
        first_choice = getpass.getpass(prompt=question)
        if not re.match(pattern, first_choice):
            printer.out("""The user password must be between 6-72 characters long and must contains at least one uppercase character, one lowercase character, one numeric digit and one special character (@#$%^&+=*!)""", printer.WARNING)
            continue;
        second_choice = getpass.getpass(prompt="Please confirm your password: ")
        if second_choice == first_choice:
            break;
        printer.out("The two passwords are different, please try again.", printer.WARNING)
    return first_choice

def format_cloud_provider(cloudprovider):
    if "aws" in cloudprovider:
        return "Amazon"
    if "openstack" in cloudprovider:
        return "OpenStack"
    if "azure" in cloudprovider:
        return "Azure "
    return cloudprovider


def print_deploy_info_line(deployment_name, deployed_instance_id, cloud_provider, location, hostname, source_type,
                       source_id, source_name, deployment_status):
    table = print_deploy_header()
    table.add_row([deployment_name, deployed_instance_id, cloud_provider, location, hostname, source_type, source_id,
               source_name, deployment_status])
    return table

def print_deploy_header():
    table = Texttable(200)
    table.set_cols_dtype(["t", "t", "t", "t", "t", "t", "t", "t", "t"])
    table.header(
        ["Deployment name", "Deployment ID", "Cloud provider", "Region", "Hostname", "Source type", "Source ID",
         "Source name", "Status"])
    return table


# Copyright 2007-2017 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

from hammr.utils.account_utils import *

from fileUtils import *

class TestK5(TestCase):
    def test_k5_should_return_cred_account_when_valid_entries(self):
        # given
        account_given = self.build_account("testName", "testLogin", "testPassword")

        # when
        account = k5(account_given)

        # then
        self.assertEqual(account.name, account_given["name"])
        self.assertEqual(account.login, account_given["login"])
        self.assertEqual(account.password, account_given["password"])


    def test_k5_should_return_none_when_missing_name(self):
        # given
        account_given = self.build_account(None, "testLogin", "testPassword")

        # when
        account = k5(account_given)

        # then
        self.assertEqual(None, account)


    def test_k5_should_return_none_when_missing_login(self):
        # given
        account_given = self.build_account("testName", None, "testPassword")

        # when
        account = k5(account_given)

        # then
        self.assertEqual(None, account)


    def test_k5_should_return_none_when_missing_password(self):
        # given
        account_given = self.build_account("testName", "testLogin", None)

        # when
        account = k5(account_given)

        # then
        self.assertEqual(None, account)


    def build_account(self, name, login, password):
        account = {}
        if name is not None: account["name"] = name
        if login is not None: account["login"] = login
        if password is not None: account["password"] = password
        return account


class TestDocker(TestCase):
    def test_docker_should_return_cred_account_when_valid_entries(self):
        # given
        account_given = self.build_account("testName", "testUrl", "testLogin", "testPassword")

        # when
        account = docker(account_given)

        # then
        self.assertEqual(account.name, account_given["name"])
        self.assertEqual(account.endpointUrl, account_given["endpointUrl"])
        self.assertEqual(account.login, account_given["login"])
        self.assertEqual(account.password, account_given["password"])


    def test_docker_should_return_none_when_missing_name(self):
        # given
        accountMocked = self.build_account(None, "testUrl", "testLogin", "testPassword")

        # when
        account = docker(accountMocked)

        # then
        self.assertEqual(None, account)


    def test_docker_should_return_none_when_missing_url(self):
        # given
        accountMocked = self.build_account("testName", None, "testLogin", "testPassword")

        # when
        account = docker(accountMocked)

        # then
        self.assertEqual(None, account)


    def test_docker_should_return_none_when_missing_login(self):
        # given
        accountMocked = self.build_account("testName", "testUrl", None, "testPassword")

        # when
        account = docker(accountMocked)

        # then
        self.assertEqual(account, None)


    def test_docker_should_return_none_when_missing_password(self):
        # given
        accountMocked = self.build_account("testName", "testUrl", "testLogin", None)

        # when
        account = docker(accountMocked)

        # then
        self.assertEqual(account, None)


    def build_account(self, name, endpoint_url, login, password):
        account = {}
        if name is not None: account["name"] = name
        if endpoint_url is not None: account["endpointUrl"] = endpoint_url
        if login is not None: account["login"] = login
        if password is not None: account["password"] = password
        return account


class TestAzureManager(TestCase):

    def test_azure_should_return_cred_account_when_valid_arm_entries(self):
        # given
        account_given = self.build_arm_account("MyAccount", "MyTenantId", "MySubscriptionId", "MyApplicationId", "MyApplicationKey")

        # when
        account = azure(account_given)

        # then
        self.assertNotEqual(account, None)
        self.assertEqual(account.name, account_given["name"])
        self.assertEqual(account.tenantId, account_given["tenantId"])
        self.assertEqual(account.subscriptionId, account_given["subscriptionId"])
        self.assertEqual(account.applicationId, account_given["applicationId"])
        self.assertEqual(account.applicationKey, account_given["applicationKey"])

    def test_azure_should_return_none_when_missing_arm_name(self):
        # given
        accountMocked = self.build_arm_account(None, "MyTenantId", "MySubscriptionId", "MyApplicationId", "MyApplicationKey")

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_missing_arm_tenantId(self):
        # given
        accountMocked = self.build_arm_account("MyAccount", None, "MySubscriptionId", "MyApplicationId", "MyApplicationKey")

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_missing_arm_subscriptionId(self):
        # given
        accountMocked = self.build_arm_account("MyAccount", "MyTenantId", None, "MyApplicationId", "MyApplicationKey")

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_missing_arm_applicationId(self):
        # given
        accountMocked = self.build_arm_account("MyAccount", "MyTenantId", "MySubscriptionId", None, "MyApplicationKey")

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_missing_arm_applicationKey(self):
        # given
        accountMocked = self.build_arm_account("MyAccount", "MyTenantId", "MySubscriptionId",  "MyApplicationId", None)

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def build_arm_account(self, name, tenantId, subscriptionId, applicationID, applicationKey):
        account = {}
        if name is not None: account["name"] = name
        if tenantId is not None: account["tenantId"] = tenantId
        if subscriptionId is not None: account["subscriptionId"] = subscriptionId
        if applicationID is not None: account["applicationId"] = applicationID
        if applicationKey is not None: account["applicationKey"] = applicationKey
        return account

    def test_azure_should_return_cred_account_when_valid_azure_classic_entries(self):
        # given
        publishSettingsFileRelativePath = findRelativePathFor("tests/integration/data/pk.pem")
        account_given = self.build_azure_classic_account("MyAccount", publishSettingsFileRelativePath)

        # when
        account = azure(account_given)

        # then
        self.assertNotEqual(account, None)
        self.assertEqual(account.name, account_given["name"])
        self.assertEqual(account.publishsettings, account_given["publishsettings"])

    def test_azure_should_return_none_when_missing_azure_classic_name(self):
        # given
        accountMocked = self.build_azure_classic_account(None, "publishsettings")

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_missing_azure_classic_publishsettings(self):
        # given
        accountMocked = self.build_azure_classic_account("MyAccount", None)

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def test_azure_should_return_none_when_azure_classic_publishsettings_file_not_found(self):
        # given
        accountMocked = self.build_azure_classic_account("MyAccount", 'tests/myNonExistingFile.publishsettings')

        # when
        account = azure(accountMocked)

        # then
        self.assertEqual(account, None)

    def build_azure_classic_account(self, name, publishsettings):
        account = {}
        if name is not None: account["name"] = name
        if publishsettings is not None: account["publishsettings"] = publishsettings
        return account


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer, progressbar_widget, download_utils
from hammr.utils import *
from uforge.objects.uforge import *
from hammr.utils.hammr_utils import *
from hammr.utils.deployment_utils import *
import shlex
import time
from progressbar import AnimatedMarker, Bar, BouncingBar, Counter, ETA, \
    FileTransferSpeed, FormatLabel, Percentage, \
    ProgressBar, ReverseBar, RotatingMarker, \
    SimpleProgress, Timer, UnknownLength


class Deploy(Cmd, CoreGlobal):
    """Displays all the deployments and instances information"""
    cmd_name = "deploy"
    pbar = None

    def __init__(self):
        super(Deploy, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                  description="Displays all the deployments and instances information")
        return doParser

    def do_list(self, args):
        try:
            printer.out("Getting all deployments for [" + self.login + "] ...")
            deployments = self.api.Users(self.login).Deployments.Getall()
            deployments = deployments.deployments.deployment

            if deployments is None or len(deployments) == 0:
                printer.out("No deployment available")
            else:
                printer.out("Deployments:")
                table = print_deploy_header()
                deployments = generics_utils.order_list_object_by(deployments, "name")
                for deployment in deployments:
                    deployment_id = deployment.applicationId
                    deployment_name = deployment.name
                    deployment_status = deployment.state
                    instances = deployment.instances.instance
                    instance = instances[-1]
                    source_type = source_id = source_name = hostname = location = cloud_provider = str(None)
                    if instance:
                        if instance.sourceSummary and type(instance.sourceSummary) == ScanSummaryLight:
                            source_type = "Scan"
                            source_id = str(extract_scannedinstance_id(instance.sourceSummary.uri))
                            source_name = instance.sourceSummary.name
                        elif instance.sourceSummary and type(instance.sourceSummary) == ApplianceSummary:
                            source_type = "Template"
                            source_id = str(generics_utils.extract_id(instance.sourceSummary.uri))
                            source_name = instance.sourceSummary.name
                        if instance.hostname:
                            hostname = instance.hostname
                        if instance.location:
                            location = instance.location.provider
                        if instance.cloudProvider:
                            cloud_provider = format_cloud_provider(instance.cloudProvider)

                    table.add_row([deployment_name, deployment_id, cloud_provider, location, hostname, source_type, source_id,
                                   source_name, deployment_status])

                print table.draw() + "\n"
                printer.out("Found " + str(len(deployments)) + " deployments")

            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()

    def arg_terminate(self):
        doParser = ArgumentParser(prog=self.cmd_name + " terminate", add_help=True,
                                  description="Terminate a deployment")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True,
                               help="id of the deployment to terminate")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--force', '-f', dest='force', required=False, action='store_true',
                              help='Terminate the deployment without asking for confirmation')
        return doParser

    def do_terminate(self, args):
        try:
            # add arguments
            doParser = self.arg_terminate()
            doArgs = doParser.parse_args(shlex.split(args))

            deployment = self.api.Users(self.login).Deployments(doArgs.id).Get()
            if doArgs.force or generics_utils.query_yes_no("Do you really want to terminate deployment with id '" + str(
                    doArgs.id) + "' named '" + deployment.name + "'"):
                # When terminating a running deployment, we stop if the status goes to on-fire.
                # But when terminating an on-fire deployment we stop if it is terminated.
                # So we need to get the status before invoking the terminate.
                status = self.api.Users(self.login).Deployments(doArgs.id).Status.Getdeploystatus()
                initial_status = status.message
                self.api.Users(self.login).Deployments(doArgs.id).Terminate()
                printer.out("Terminating the deployment")
                bar = ProgressBar(widgets=[BouncingBar()], maxval=UnknownLength)
                bar.start()
                i = 1
                while (True):
                    deployment = self.get_deployment(doArgs.id)
                    if deployment is None:
                        break;
                    if initial_status != "on-fire":
                        if deployment.state == "on-fire":
                            # If the deployment went from running to on-fire, we stop and print an error message
                            break
                    time.sleep(1)
                    bar.update(i)
                    i += 2
                bar.finish()

                if deployment:
                    printer.out("Could not terminate the deployment.", printer.ERROR)
                    status = self.api.Users(self.login).Deployments(doArgs.id).Status.Getdeploystatus()
                    if status.message == "on-fire" and status.detailedError:
                        printer.out(status.detailedErrorMsg, printer.ERROR)
                    return 1
                else:
                    printer.out("Deployment terminated", printer.OK)

            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_terminate()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_terminate(self):
        doParser = self.arg_terminate()
        doParser.print_help()

    def get_deployment(self, searched_deploy_id):
        deployments = self.api.Users(self.login).Deployments.Getall()
        deployments = deployments.deployments.deployment

        if deployments is None or len(deployments) == 0:
            return None
        else:
            for deployment in deployments:
                deployment_id = deployment.applicationId
                if deployment_id == searched_deploy_id:
                    return deployment
        return None


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import shlex
from hurry.filesize import size

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from argparse import RawTextHelpFormatter
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer
from hammr.utils.hammr_utils import *



class Os(Cmd, CoreGlobal):
    """List all the OSes the user has access to. Includes, name, version, architecture and release date. You can also search for available packages"""

    cmd_name="os"

    def __init__(self):
        super(Os, self).__init__()


    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name+" list", add_help = True, description="Displays all the operating systems available to use by the use")
        return doParser

    def do_list(self, args):
        try:
            #call UForge API
            printer.out("Getting distributions for ["+self.login+"] ...")
            distributions = self.api.Users(self.login).Distros.Getall()
            distributions = distributions.distributions
            if distributions is None or not hasattr(distributions, "distribution"):
                printer.out("No distributions available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t","t","t","t","t", "t"])
                table.header(["Id", "Name", "Version", "Architecture", "Release Date", "Profiles"])
                distributions = generics_utils.order_list_object_by(distributions.distribution, "name")
                for distribution in distributions:
                    profiles = self.api.Distributions(distribution.dbId).Profiles.Getall()
                    profiles = profiles.distribProfiles.distribProfile
                    if len(profiles) > 0:
                        profile_text=""
                        for profile in profiles:
                            profile_text+=profile.name+"\n"
                        table.add_row([distribution.dbId, distribution.name, distribution.version, distribution.arch, distribution.releaseDate.strftime("%Y-%m-%d %H:%M:%S") if distribution.releaseDate is not None else "", profile_text])
                    else:
                        table.add_row([distribution.dbId, distribution.name, distribution.version, distribution.arch, distribution.releaseDate.strftime("%Y-%m-%d %H:%M:%S") if distribution.releaseDate is not None else "", "-"])
                print table.draw() + "\n"
                printer.out("Found "+str(len(distributions))+" distributions")
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()



    def arg_search(self):
        doParser = ArgumentParser(prog=self.cmd_name+" search", add_help = True, description="Search packages from an OS", formatter_class=RawTextHelpFormatter)
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="Os id")
        mandatory.add_argument('--pkg', dest='pkg', required=True, help='''\
                Regular expression of the package:\n\
                "string" : search all packages which contains "string"\n\
                "*string": search all packages which end with "string"\n\
                "string*": search all packages which start with "string"''')
        return doParser

    def do_search(self, args):
        try:
            #add arguments
            doParser = self.arg_search()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            #call UForge API
            printer.out("Search package '"+doArgs.pkg+"' ...")
            distribution = self.api.Distributions(doArgs.id).Get()
            printer.out("for OS '"+distribution.name+"', version "+distribution.version)
            pkgs = self.api.Distributions(distribution.dbId).Pkgs.Getall(Query="name=="+doArgs.pkg)
            pkgs = pkgs.pkgs.pkg
            if pkgs is None or len(pkgs) == 0:
                printer.out("No package found")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t","t","t","t","t","t","t"])
                table.header(["Name", "Version", "Arch", "Release", "Build date", "Size", "FullName"])
                pkgs = generics_utils.order_list_object_by(pkgs, "name")
                for pkg in pkgs:
                    table.add_row([pkg.name, pkg.version, pkg.arch, pkg.release, pkg.pkgBuildDate.strftime("%Y-%m-%d %H:%M:%S"), size(pkg.size), pkg.fullName])
                print table.draw() + "\n"
                printer.out("Found "+str(len(pkgs))+" packages")
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_search()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_search(self):
        doParser = self.arg_search()
        doParser.print_help()




# Copyright 2007-2017 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os.path

def findRelativePathFor(myUnitTestDataFile):
    # When Unit Tests are launched from pycher, the path is the directory of the unit test file
    if os.path.isfile('../../../' + myUnitTestDataFile):
        return '../../../' + myUnitTestDataFile
    # When Unit Tests are launched from CI, the path is the project directory
    return myUnitTestDataFile


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import shlex

import time
import os.path
import ntpath
import tarfile
import shutil

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer
from progressbar import Bar, Percentage, ProgressBar
from hammr.utils.hammr_utils import *
from hammr.utils.bundle_utils import *
from hammr.utils import constants
from uforge.objects.uforge import *
from hurry.filesize import size
from os.path import realpath

class Bundle(Cmd, CoreGlobal):
    """Create a bundle (mysoftware) on UForge based on your configuration file. List, delete, clone or export/import existing bundles. Validate your configuration file before creating your bundle"""

    cmd_name="bundle"

    def __init__(self):
        super(Bundle, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name+" list", add_help = True, description="Lists all the bundles that have been registered in the UForge server")
        return doParser

    def do_list(self, args):
        try:
            #call UForge API
            printer.out("Getting all your bundles ...")
            bundles = self.api.Users(self.login).Mysoftware.Getall()
            bundles = bundles.mySoftwareList.mySoftware
            if bundles is None or len(bundles) == 0:
                printer.out("No bundles available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t","t","t", "t","t","t","t"])
                table.header(["Id", "Name", "Version", "Description", "Category", "Size", "Imported"])
                bundles = generics_utils.order_list_object_by(bundles, "name")
                for bundle in bundles:
                    category = ""
                    if bundle.category is not None:
                        category = bundle.category.name
                    table.add_row([bundle.dbId, bundle.name, bundle.version, bundle.description, category, size(bundle.size), "X" if bundle.imported else ""])
                print table.draw() + "\n"
                printer.out("Found "+str(len(bundles))+" bundles")

            return 0
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()


    def arg_export(self):
        doParser = ArgumentParser(prog=self.cmd_name+" export", add_help = True, description="Exports a bundle by creating an archive (compressed tar file) that includes the bundle configuration file")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the bundle to export")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--file', dest='file', required=False, help="destination path where to store the bundle configuration file on the local filesystem")
        optional.add_argument('--outputFormat', dest='output_format', required=False, help="output format (yaml or json) of the bundle file to export (yaml is the default one)")
        return doParser

    def do_export(self, args):
        try:
            #add arguments
            doParser = self.arg_export()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            #call UForge API
            printer.out("Exporting bundle with id ["+doArgs.id+"] :")
            mySoftware = self.api.Users(self.login).Mysoftware(doArgs.id).Get()
            if mySoftware is None or type(mySoftware) is not MySoftware:
                printer.out("No bundle")
            else:
                output_format="yaml"
                if doArgs.output_format is not None:
                   output_format=doArgs.output_format
                mySoftwareExport = self.api.Users(self.login).Mysoftware(mySoftware.dbId).Exports().Export(output_format)
                status = mySoftwareExport.status
                progress = ProgressBar(widgets=[Percentage(), Bar()], maxval=100).start()
                while not (status.complete or status.error):
                    progress.update(status.percentage)
                    status = self.api.Users(self.login).Mysoftware(mySoftware.dbId).Exports(mySoftwareExport.dbId).Status.Get()
                    time.sleep(2)
                progress.finish()
                if status.error:
                    printer.out("Export error: "+status.message+"\n"+status.errorMessage, printer.ERROR)
                    if status.detailedError:
                        printer.out(status.detailedErrorMsg)
                else:
                    printer.out("Downloading archive...")
                    if doArgs.file is None:
                        file = open(mySoftware.name+".tar.gz", "w")
                    else:
                        file = open(doArgs.file, "w")
                    self.api.Users(self.login).Mysoftware(mySoftware.dbId).Exports(mySoftwareExport.dbId).Downloads.Download(streamingResponseFile=file)
                    file.close()

                    #Delete export archive on the server
                    self.api.Users(self.login).Mysoftware(mySoftware.dbId).Exports(mySoftwareExport.dbId).Delete()

                    printer.out("Download complete of file ["+file.name+"]", printer.OK)
            return 0
        except IOError as e:
            printer.out("File error: "+str(e), printer.ERROR)
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_export()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_export(self):
        doParser = self.arg_export()
        doParser.print_help()


    def arg_import(self):
        doParser = ArgumentParser(prog=self.cmd_name+" import", add_help = True, description="Creates a bundle from an archive")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True, help="the path of the archive")
        optional = doParser.add_argument_group("optional arguments")
        return doParser

    def do_import(self, args):
        try:
            #add arguments
            doParser = self.arg_import()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            #call UForge API
            return self.import_bundle(doArgs.file, True)
        except ArgumentParserError as e:
            printer.out("In Arguments: "+str(e)+"\n", printer.ERROR)
            self.help_import()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_import(self):
        doParser = self.arg_import()
        doParser.print_help()


    def arg_validate(self):
        doParser = ArgumentParser(prog=self.cmd_name+" validate", add_help = True, description="Validates the syntax of a bundle configuration file")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True, help="the yaml/json bundle configuration file")
        return doParser

    def do_validate(self, args):
        try:
            #add arguments
            doParser = self.arg_validate()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2
            bundle=validate_bundle(file)
            if bundle is None:
                return 2
            print "OK : Syntax of bundle file [" + realpath(file) + "] is ok"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_validate()

    def help_validate(self):
        doParser = self.arg_validate()
        doParser.print_help()


    def arg_create(self):
        doParser = ArgumentParser(prog=self.cmd_name+" create", add_help = True, description="Create a new bundle and save to the UForge server")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True, help="yaml/json file containing the bundle content")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--archive-path', dest='archive_path', required=False, help="path of where to store the archive of the created bundle. If provided hammr, creates an archive of the created bundle, equivalent to running bundle export")
        return doParser

    def do_create(self, args):
        try:
            #add arguments
            doParser = self.arg_create()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            #--
            #get file (remote or local)
            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2

            bundle = validate_bundle(file)
            if bundle is None:
                return 2
            isJsonFile= check_extension_is_json(file)
            archive_files=[]
            try:
                checkList = []
                if "files" in bundle:
                    for files in bundle["files"]:
                        checkList,archive_files = recursivelyAppendToArchive(bundle, files, "", checkList, archive_files)
                else:
                    printer.out("No files section found for bundle", printer.ERROR)
                    return 2
                if "license" in bundle and "source" in bundle["license"]:
                    #add to list of file to tar
                    file_tar_path=constants.FOLDER_BUNDLES + os.sep + generics_utils.remove_URI_forbidden_char(bundle["name"]) + os.sep + generics_utils.remove_URI_forbidden_char(ntpath.basename(bundle["license"]["source"]))
                    archive_files.append([file_tar_path,bundle["license"]["source"]])
                    #changing source path to archive related source path
                    bundle["license"]["source"]=file_tar_path
                if "sourceLogo" in bundle:
                    #add to list of file to tar
                    file_tar_path=constants.FOLDER_BUNDLES + os.sep + generics_utils.remove_URI_forbidden_char(bundle["name"]) + os.sep + generics_utils.remove_URI_forbidden_char(ntpath.basename(bundle["sourceLogo"]))
                    archive_files.append([file_tar_path,bundle["sourceLogo"]])
                    #changing source path to archive related source path
                    bundle["sourceLogo"]=file_tar_path
            except KeyError as e:
                printer.out("Error in bundle", printer.ERROR)
                return 2

            if os.path.isdir(constants.TMP_WORKING_DIR):
                #delete tmp dir
                shutil.rmtree(constants.TMP_WORKING_DIR)
            os.mkdir(constants.TMP_WORKING_DIR)

            if isJsonFile:
                fileName = constants.BUNDLE_JSON_FILE_NAME
                newFileName = constants.BUNDLE_JSON_NEW_FILE_NAME
            else:
                fileName = constants.BUNDLE_YAML_FILE_NAME
                newFileName = constants.BUNDLE_YAML_NEW_FILE_NAME

            archive_files = dump_data_in_file(bundle, archive_files, isJsonFile, fileName, newFileName)

            if doArgs.archive_path is not None:
                tar_path = doArgs.archive_path
            else:
                tar_path = constants.TMP_WORKING_DIR+os.sep+"archive.tar.gz"
            tar = tarfile.open(tar_path, "w|gz")
            for file_tar_path,file_global_path in archive_files:
                if not os.path.isdir(file_global_path):
                    file = generics_utils.get_file(file_global_path, constants.TMP_WORKING_DIR+os.sep+os.path.basename(file_global_path))
                    if file is None:
                        printer.out("Downloaded bunlde file not found", printer.ERROR)
                        return 2
                    tar.add(file, arcname=file_tar_path)
                else:
                    tar.add(file_global_path, arcname=file_tar_path)
            tar.close()

            #arhive is created, doing import
            r = self.import_bundle(tar_path, False)
            if r != 0:
                return r

            #delete tmp dir
            shutil.rmtree(constants.TMP_WORKING_DIR)
            return 0
        except OSError as e:
            printer.out("OSError: "+str(e), printer.ERROR)
        except IOError as e:
            printer.out("File error: "+str(e), printer.ERROR)
        except ArgumentParserError as e:
            printer.out("In Arguments: "+str(e), printer.ERROR)
            self.help_create()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_create(self):
        doParser = self.arg_create()
        doParser.print_help()


    def import_bundle(self, file, isImport):
        try:
            if isImport:
                printer.out("Importing bundle from ["+file+"] archive ...")
            else:
                if constants.TMP_WORKING_DIR in str(file):
                    printer.out("Creating bundle from temporary ["+file+"] archive ...")
                else:
                    printer.out("Creating bundle from ["+file+"] archive ...")
            file = open(file, "r")

            bundleImport = self.api.Users(self.login).Bundleimports.Import(Imported=isImport)

            if bundleImport is None:
                if isImport:
                    printer.out("error importing bundle", printer.ERROR)
                else:
                    printer.out("error creating bundle", printer.ERROR)
                return 2
            else:
                status = self.api.Users(self.login).Bundleimports(bundleImport.dbId).Uploads.Upload(file)
                progress = ProgressBar(widgets=[Percentage(), Bar()], maxval=100).start()
                while not (status.complete or status.error):
                    progress.update(status.percentage)
                    status = self.api.Users(self.login).Bundleimports(bundleImport.dbId).Status.Get()
                    time.sleep(2)
                progress.finish()
                if status.error:
                    if isImport:
                        printer.out("Bundle import: "+status.message+"\n"+status.errorMessage, printer.ERROR)
                        if status.detailedError:
                            printer.out(status.detailedErrorMsg)
                    else:
                        printer.out("Bundle create: "+status.message+"\n"+status.errorMessage, printer.ERROR)
                else:
                    if isImport:
                        printer.out("Bundle import: DONE", printer.OK)
                    else:
                        printer.out("Bundle create: DONE", printer.OK)

                    #get bundle import
                    bundleImport = self.api.Users(self.login).Bundleimports(bundleImport.dbId).Get()
                    printer.out("Bundle URI: "+bundleImport.referenceUri)
                    printer.out("Bundle Id : "+generics_utils.extract_id(bundleImport.referenceUri))

                return 0
        except IOError as e:
            printer.out("File error: "+str(e), printer.ERROR)
            return 2
        except Exception as e:
            return handle_uforge_exception(e)


    def arg_delete(self):
        doParser = ArgumentParser(prog=self.cmd_name+" delete", add_help = True, description="Deletes an existing bundle")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the bundle to delete")
        return doParser

    def do_delete(self, args):
        try:
            #add arguments
            doParser = self.arg_delete()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            #call UForge API
            printer.out("Searching bundle with id ["+doArgs.id+"] ...")
            myBundle = self.api.Users(self.login).Mysoftware(doArgs.id).Get()
            if myBundle is None or type(myBundle) is not MySoftware:
                printer.out("Bundle not found", printer.WARNING)
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t","t","t", "t","t", "t"])
                table.header(["Id", "Name", "Version", "Description", "Size", "Imported"])
                table.add_row([myBundle.dbId, myBundle.name, myBundle.version, myBundle.description, size(myBundle.size), "X" if myBundle.imported else ""])
                print table.draw() + "\n"
                if generics_utils.query_yes_no("Do you really want to delete bundle with id "+str(myBundle.dbId)):
                    self.api.Users(self.login).Mysoftware(myBundle.dbId).Delete()
                    printer.out("Bundle deleted", printer.OK)


        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_delete(self):
        doParser = self.arg_delete()
        doParser.print_help()


    def arg_clone(self):
        doParser = ArgumentParser(prog=self.cmd_name+" clone", add_help = True, description="Clones the bundle. The clone is copying the meta-data of the bundle")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the bundle to clone")
        mandatory.add_argument('--name', dest='name', required=True, help="the name to use for the new cloned bundle")
        mandatory.add_argument('--version', dest='version', required=True, help="the version to use for the cloned bundle")
        return doParser

    def do_clone(self, args):
        try:
            #add arguments
            doParser = self.arg_clone()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            #call UForge API
            printer.out("Clonnig bundle with id ["+doArgs.id+"] ...")
            software = mySoftware()
            software.name = doArgs.name
            software.version = doArgs.version
            rMySoftware = self.clone_mySoftware(doArgs.id, software)
            if type(rMySoftware) is MySoftware:
                printer.out("Clonned successfully", printer.OK)
            else:
                printer.out("Clone error", printer.ERROR)
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_clone()
        except Exception as e:
            return handle_uforge_exception(e)

    def clone_mySoftware(self, id, software):
        return self.api.Users(self.login).Mysoftware(id).Clone.Clone(software)

    def help_clone(self):
        doParser = self.arg_clone()
        doParser.print_help()


    def arg_categories(self):
        doParser = ArgumentParser(prog=self.cmd_name+" categories", add_help = True, description="Lists all the categories available for bundles.")
        return doParser

    def do_categories(self, args):
        try:
            #call UForge API
            printer.out("Getting all categories available for bundles ...")
            orgId = (self.api.Users(self.login).Orgs().Getall()).orgs.org[0].dbId
            categories = self.api.Orgs(orgId).Categories.Getall(Query="type=='PROJECT'")
            categories = generics_utils.order_list_object_by(categories.categories.category, "name")
            if categories is None or len(categories) == 0:
                printer.out("No categories available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t"])
                table.header(["Name"])
                for category in categories:
                    table.add_row([category.name])
                print table.draw() + "\n"
                printer.out("Found "+str(len(categories))+" categories")

            return 0
        except Exception as e:
            return handle_uforge_exception(e)

    def help_categories(self):
        doParser = self.arg_list()
        doParser.print_help()


# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from mock import patch
from uforge.application import Api
from hammr.utils import constants
from uforge.objects.uforge import *
from uforge.objects import uforge
import datetime
from hammr.utils.deployment_utils import *


class TestDeployUtils(TestCase):

    def test_build_deployment_openstack_returns_None_when_file_incomplete(self):
        # Given
        file = "tests/integration/data/deploy_openstack_incomplete.yml"

        # When
        return_value = build_deployment_openstack(file, None, None, None)

        #Then
        self.assertEquals(return_value, None)

    def testbuild_deployment_amazon_returns_None_when_file_incomplete(self):
        # Given
        file = "tests/integration/data/deploy_aws_incomplete.yml"

        # When
        return_value = build_deployment_amazon(file)

        #Then
        self.assertEquals(return_value, None)

    def testbuild_deployment_azure_returns_None_when_file_incomplete(self):
        # Given
        file = "tests/integration/data/deploy_azure_incomplete.yml"

        # When
        return_value = build_deployment_azure(file)

        #Then
        self.assertEquals(return_value, None)





# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from mock import patch
from uforge.application import Api
from uforge.objects import uforge
from uforge.objects.uforge import *

from hammr.commands.deploy import deploy
from hammr.utils import constants

class TestDeployList(TestCase):
    # URL: users/{uid}/deployments/{did}/status
    @patch('uforge.application.Api._Users._Deployments.Getall')
    @patch('texttable.Texttable.add_row')
    def test_do_list_gives_correct_number_of_deployments(self, mock_table_add_row, mock_api_deployments_getall):
        # given
        i = deploy.Deploy()
        i.api = Api("url", username="username", password="password", headers=None,
                    disable_ssl_certificate_validation=False, timeout=constants.HTTP_TIMEOUT)
        i.login = "login"
        i.password = "password"
        self.create_deployments(mock_api_deployments_getall)

        # when
        i.do_list(None)

        # then
        self.assertEquals(mock_table_add_row.call_count, 1)

    def create_deployments(self,mock_api_deployments_getall):
        new_deployments = uforge.deployments()
        new_deployments.deployments = pyxb.BIND()

        newdeployment = self.create_deployment()
        new_deployments.deployments.append(newdeployment)

        mock_api_deployments_getall.return_value = new_deployments

    def create_deployment(self):
        deployment = Deployment()
        deployment.name = "DeploymentName"
        deployment.applicationId = "id123456789"
        deployment.state = "running"

        myinstance = Instance()
        myinstance.cores = "1"
        myinstance.memory = "1024"
        myinstance.hostname = "example.com"
        myLocation = Location()
        myLocation.provider = "myprovider"
        myinstance.location = myLocation

        deployment.instances = pyxb.BIND()
        deployment.instances._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Instances')
        deployment.instances.append(myinstance)

        return deployment



# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from mock import patch
from uforge.application import Api
from hammr.utils import constants
from uforge.objects.uforge import *
from uforge.objects import uforge
import datetime
from hammr.commands.image import image
from test_deploy_aws import TestDeployAWS

class TestDeployAzure(TestCase):
    @patch('uforge.application.Api._Users._Pimages.Getall')
    @patch('uforge.application.Api._Users._Deployments._Status.Getdeploystatus')
    @patch('uforge.application.Api._Users._Appliances._Images._Pimages._Deploys.Deploy')
    @patch('uforge.application.Api._Users._Appliances.Get')
    @patch('uforge.application.Api._Users._Deployments.Get')
    def test_do_deploy_azure_return_0_when_status_is_running(self, mock_get_deployment, mock_app_get, mock_api_deploy,
                                                        mock_get_deploy_status, mock_api_pimg_getall_for_app):
        # given
        i = TestDeployAWS.prepare_image()
        args = self.prepare_image_deploy_command_azure(1234)

        self.prepare_mock_deploy_azure(mock_get_deployment, mock_api_deploy)
        TestDeployAWS.prepare_mock_app_get(mock_app_get)
        self.prepare_mock_api_pimg_getall_for_app_azure(mock_api_pimg_getall_for_app)
        TestDeployAWS.prepare_mock_deploy_status_running(mock_get_deploy_status)

        # when
        deploy_return = i.do_deploy(args)

        # then
        self.assertEquals(deploy_return, 0)

    @patch('uforge.application.Api._Users._Pimages.Getall')
    @patch('uforge.application.Api._Users._Deployments._Status.Getdeploystatus')
    @patch('uforge.application.Api._Users._Scannedinstances._Scans._Images._Pimages._Deploys.Deploy')
    @patch('uforge.application.Api._Users._Scannedinstances._Scans.Get')
    @patch('uforge.application.Api._Users._Deployments.Get')
    def test_do_deploy_azure_a_scan_return_0_when_status_is_running(self, mock_get_deployment, mock_scan_get, mock_scan_deploy,
                                                        mock_get_deploy_status, prepare_mock_api_pimg_getall_for_scan):
        # given
        i = TestDeployAWS.prepare_image()
        args = self.prepare_image_deploy_command_azure(1234)

        self.prepare_mock_deploy_azure(mock_get_deployment, mock_scan_deploy)
        TestDeployAWS.prepare_mock_scan_get(mock_scan_get)
        self.prepare_mock_api_pimg_getall_for_scan_azure(prepare_mock_api_pimg_getall_for_scan)
        TestDeployAWS.prepare_mock_deploy_status_running(mock_get_deploy_status)

        # when
        deploy_return = i.do_deploy(args)

        # then
        self.assertEquals(deploy_return, 0)

    @patch('uforge.application.Api._Users._Pimages.Getall')
    @patch('uforge.application.Api._Users._Deployments._Status.Getdeploystatus')
    @patch('uforge.application.Api._Users._Appliances._Images._Pimages._Deploys.Deploy')
    @patch('uforge.application.Api._Users._Appliances.Get')
    @patch('uforge.application.Api._Users._Deployments.Get')
    def test_do_deploy_azure_return_1_when_status_is_onfire(self, mock_get_deployment, mock_app_get, mock_api_deploy,
                                                          mock_get_deploy_status, mock_api_pimg_getall_for_app):
        # given
        i = TestDeployAWS.prepare_image()
        args = self.prepare_image_deploy_command_azure(1234)

        self.prepare_mock_deploy_azure(mock_get_deployment, mock_api_deploy)
        TestDeployAWS.prepare_mock_app_get(mock_app_get)
        self.prepare_mock_api_pimg_getall_for_app_azure(mock_api_pimg_getall_for_app)
        TestDeployAWS.prepare_mock_deploy_status_onfire(mock_get_deploy_status)

        # when
        deploy_return = i.do_deploy(args)

        # then
        self.assertEquals(deploy_return, 1)

    def get_deployment_azure(self):
        deployment = Deployment()
        deployment.name = "DeploymentName"
        deployment.applicationId = "id123456789"

        myinstance = InstanceAzureResourceManager()
        myinstance.cores = "1"
        myinstance.memory = "1024"
        myinstance.hostname = "example.com"
        myinstance.userName = "myName"
        myinstance.userSshKey = "mySshKey"

        myLocation = Location()
        myLocation.provider = "myprovider"
        myinstance.location = myLocation
        myinstance.cloudProvider = "azurearm"

        deployment.instances = pyxb.BIND()
        deployment.instances._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Instances')
        deployment.instances.append(myinstance)

        return deployment

    def prepare_image_deploy_command_azure(self, id):
         args = "--file tests/integration/data/deploy_azure.yml --publish-id %s" % (id)
         return args

    def prepare_azure_pimages_from_app(self):
        new_pimages = uforge.publishImages()
        new_pimages.publishImages = pyxb.BIND()

        newImage = PublishImageAzureResourceManager()
        newImage.dbId = 1234
        newImage.imageUri = 'users/guest/appliances/5/images/1234'
        newImage.applianceUri = 'users/guest/appliances/5'
        newImage.status = "complete"
        newImage.status.complete = True
        newImage.targetFormat = uforge.targetFormat()
        newImage.targetFormat.name = "Azure ARM"

        new_pimages.publishImages.append(newImage)

        return new_pimages

    def prepare_pimages_from_scan_azure(self):
        new_pimages = uforge.publishImages()
        new_pimages.publishImages = pyxb.BIND()

        newImage = PublishImageAzureResourceManager()
        newImage.dbId = 1234
        newImage.imageUri = 'users/guest/scannedinstances/5/scans/12/images/1234'
        newImage.status = "complete"
        newImage.status.complete = True
        newImage.targetFormat = uforge.targetFormat()
        newImage.targetFormat.name = "Azure ARM"

        new_pimages.publishImages.append(newImage)

        return new_pimages

    def prepare_mock_deploy_azure(self, mock_get_deployment, mock_api_deploy):
        deployment = self.get_deployment_azure()
        mock_api_deploy.return_value = deployment
        mock_get_deployment.return_value = deployment

    def prepare_mock_api_pimg_getall_for_app_azure(self, mock_api_pimg_getall_for_app):
        new_pimages = self.prepare_azure_pimages_from_app()
        mock_api_pimg_getall_for_app.return_value = new_pimages

    def prepare_mock_api_pimg_getall_for_scan_azure(self, mock_api_pimg_getall_for_scan):
        new_pimages = self.prepare_pimages_from_scan_azure()
        mock_api_pimg_getall_for_scan.return_value = new_pimages



# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

try:
    from termcolor import colored
except ImportError:
    def colored(string, a=None, b=None, attrs=None):
        return string
try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

import argparse
import getpass
import os
import sys
import shlex


from ussclicore.cmd import Cmd, CmdUtils
from ussclicore.argumentParser import CoreArgumentParser, ArgumentParser, ArgumentParserError
from ussclicore.utils import generics_utils
from ussclicore.utils import printer
import commands
from uforge.application import Api, checkUForgeCompatible
from utils import constants, hammr_utils

class CmdBuilder(object):
    @staticmethod
    def generateCommands(class_):
        # Create subCmds if not exist
        if not hasattr(class_, 'subCmds'):
            class_.subCmds = {}
            # Add commands
        user = commands.user.User()
        class_.subCmds[user.cmd_name] = user
        template = commands.template.Template()
        class_.subCmds[template.cmd_name] = template
        _os = commands.os.Os()
        class_.subCmds[_os.cmd_name] = _os
        format = commands.format.Format()
        class_.subCmds[format.cmd_name] = format
        image = commands.image.Image()
        class_.subCmds[image.cmd_name] = image
        account = commands.account.Account()
        class_.subCmds[account.cmd_name] = account
        bundle = commands.bundle.Bundle()
        class_.subCmds[bundle.cmd_name] = bundle
        scan = commands.scan.Scan()
        class_.subCmds[scan.cmd_name] = scan
        quota = commands.quota.Quota()
        class_.subCmds[quota.cmd_name] = quota
        platform = commands.platform.Platform()
        class_.subCmds[platform.cmd_name] = platform
        deploy = commands.deploy.Deploy()
        class_.subCmds[deploy.cmd_name] = deploy

## Main cmd
class Hammr(Cmd):
    #    subCmds = {
    #        'tools': CmdUtils
    #    }
    def __init__(self):
        super(Hammr, self).__init__()
        self.prompt = 'hammr> '

    def do_exit(self, args):
        return True

    def do_quit(self, args):
        return True

    def arg_batch(self):
        doParser = ArgumentParser("batch", add_help = True, description="Execute hammr batch command from a file (for scripting)")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True, help="hammr batch file commands")
        return doParser

    def do_batch(self, args):
        try:
            doParser = self.arg_batch()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            with open(doArgs.file) as f:
                for line in f:
                    try:
                        self.run_commands_at_invocation([line])
                    except:
                        printer.out("bad command '"+line+"'", printer.ERROR)
                    print "\n"
        except IOError as e:
            printer.out("File error: "+str(e), printer.ERROR)
            return
        except ArgumentParserError as e:
            printer.out("In Arguments: "+str(e), printer.ERROR)
            self.help_batch()

    def help_batch(self):
        doParser = self.arg_batch()
        doParser.print_help()

    def compatibility_verbose(self):
        try:
            compatible, serviceStatusVersion = checkUForgeCompatible(api)
            if not compatible:
                printer.out("Sorry but this version of Hammr (version = '" + str(
                    constants.VERSION) + "') is not compatible with the version of UForge (version = '" + str(
                    serviceStatusVersion) + "').", printer.ERROR)
                printer.out(
                    "Please refer to 'Install Compatibility' section in the documentation to learn how to install a compatible version of Hammr.",
                    printer.ERROR)
                sys.exit(2)

        except Exception as e:
            hammr_utils.print_uforge_exception(e)
            sys.exit(2)

    def cmdloop(self, args):
        self.compatibility_verbose()
        if len(args):
            code = self.run_commands_at_invocation([str.join(' ', args)])
            sys.exit(code)
        else:
            self._cmdloop()

def generate_base_doc(app, hamm_help):
    myactions=[]
    cmds= sorted(app.subCmds)
    for cmd in cmds:
        myactions.append(argparse._StoreAction(
            option_strings=[],
            dest=str(cmd),
            nargs=None,
            const=None,
            default=None,
            type=str,
            choices=None,
            required=False,
            help=str(app.subCmds[cmd].__doc__),
            metavar=None))
    return myactions

def set_globals_cmds(subCmds):
    for cmd in subCmds:
        if hasattr(subCmds[cmd], 'set_globals'):
            subCmds[cmd].set_globals(api, login, password)
            if hasattr(subCmds[cmd], 'subCmds'):
                set_globals_cmds(subCmds[cmd].subCmds)

def check_credfile(credfile):
    printer.out("Checking given credentials file: " + credfile, printer.INFO)
    if os.path.isfile(credfile):
        return credfile
    return None

def check_default_credfile():
    printer.out("Checking default credentials file...", printer.INFO)
    credfile_yaml=hammr_utils.get_hammr_dir()+os.sep+"credentials.yml"
    if os.path.isfile(credfile_yaml):
        return credfile_yaml
    credfile_json=hammr_utils.get_hammr_dir()+os.sep+"credentials.json"
    if os.path.isfile(credfile_json):
        return credfile_json
    return None

#Generate hammr base command + help base command
CmdBuilder.generateCommands(Hammr)
app = Hammr()
myactions=generate_base_doc(app, hamm_help="")

# Args parsing
mainParser = CoreArgumentParser(add_help=False)
CoreArgumentParser.actions=myactions
mainParser.add_argument('-a', '--url', dest='url', type=str, help='the UForge server URL endpoint to use', required = False)
mainParser.add_argument('-u', '--user', dest='user', type=str, help='the user name used to authenticate to the UForge server', required = False)
mainParser.add_argument('-p', '--password', dest='password', type=str, help='the password used to authenticate to the UForge server', required = False)
mainParser.add_argument('-c', '--credentials', dest='credentials', type=str, help='the credential file used to authenticate to the UForge server (default to ~/.hammr/credentials.yml or ~/.hammr/credentials.json)', required = False)
mainParser.add_argument('-v', action='version', help='displays the current version of the hammr tool', version="%(prog)s version '"+constants.VERSION+"'")
mainParser.add_argument('-h', '--help', dest='help', action='store_true', help='show this help message and exit', required = False)
mainParser.set_defaults(help=False)
mainParser.add_argument('cmds', nargs='*', help='Hammr cmds')
mainArgs, unknown = mainParser.parse_known_args()

if mainArgs.help and not mainArgs.cmds:
    mainParser.print_help()
    exit(0)

if mainArgs.url is not None:
    url=mainArgs.url

if mainArgs.user is not None:
    if not mainArgs.password:
        mainArgs.password = getpass.getpass()
    username=mainArgs.user
    password=mainArgs.password
    sslAutosigned=True
else:
    if mainArgs.credentials is not None:
        credfile=mainArgs.credentials
        credpath=check_credfile(credfile)
        if credpath is None:
            printer.out("credentials file '" + credfile + "' not found\n", printer.ERROR)
            exit(1)
    else:
        credpath=check_default_credfile()
        if credpath is None:
            printer.out("credentials file 'credentials.yml' or 'credentials.json' not found\n", printer.ERROR)
            exit(1)

    printer.out("no username nor password provided on command line, trying credentials file", printer.INFO)

    printer.out("Using credentials file: " + credpath, printer.INFO)
    try:
        data = hammr_utils.load_data(credpath)
        if mainArgs.user:
            username=mainArgs.user
        elif "user" in data:
            username=data["user"]
        else:
            printer.out("username not found in credentials file", printer.ERROR)
        if mainArgs.password:
            password=mainArgs.password
        elif "password" in data:
            password=data["password"]
        else:
            printer.out("password not found in credentials file", printer.ERROR)
        if mainArgs.url:
            url=mainArgs.url
        elif "url" in data:
            url=data["url"]
        else:
            printer.out("url not found in credentials file", printer.ERROR)
        printer.out("Using url " + url, printer.INFO)
        if "acceptAutoSigned" in data:
            sslAutosigned=data["acceptAutoSigned"]
        else:
            sslAutosigned=True
    except ValueError as e:
        printer.out("parsing error in credentials file: "+str(e), printer.ERROR)
    except IOError as e:
        printer.out("File error in credentials file: "+str(e), printer.ERROR)
    except Exception as e:
        hammr_utils.print_uforge_exception(e)
        exit(1)

#UForge API instanciation
api = Api(url, username = username, password = password, headers = None, disable_ssl_certificate_validation = sslAutosigned, timeout = constants.HTTP_TIMEOUT)

if generics_utils.is_superviser_mode(username):
    login = generics_utils.get_target_username(username)
else:
    login = username

set_globals_cmds(app.subCmds)

if mainArgs.help and len(mainArgs.cmds)>=1:
    argList=mainArgs.cmds + unknown;
    argList.insert(len(mainArgs.cmds)-1, "help")
    app.cmdloop(argList)
elif mainArgs.help:
    app.cmdloop(mainArgs.cmds + unknown + ["-h"])
else:
    app.cmdloop(mainArgs.cmds + unknown)



__author__ = 'UshareSoft'

import unittest
import json
import yaml
from mock import patch

from hammr.utils import hammr_utils

class TestFiles(unittest.TestCase):

    def test_pythonObjectFromYamlParsingShouldBeTheSameAsJsonParsing(self):
        #Given
        jsonPath = "tests/integration/data/test-parsing.json"
        yamlPath = "tests/integration/data/test-parsing.yml"
        #When
        jsonData = json.load(open(jsonPath))
        yamlData = yaml.load(open(yamlPath))
        #Then
        self.assertEqual(jsonData, yamlData, 'Error : python object made from json parsing is different from yaml parsing')


    @patch("hammr.utils.hammr_utils.validate_configurations_file")
    def test_check_extension_and_validate_json(self, mock_method):
        #Given
        jsonPath = "test.json"
        #When
        hammr_utils.validate(jsonPath)
        #Then
        mock_method.assert_called_with(jsonPath, isJson=True)


    @patch("hammr.utils.hammr_utils.validate_configurations_file")
    def test_check_extension_and_validate_yaml(self, mock_method):
        #Given
        yamlPath = "test.yml"
        #When
        hammr_utils.validate(yamlPath)
        #Then
        mock_method.assert_called_with(yamlPath, isJson=False)


if __name__ == '__main__':
    unittest.main()





# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
import shlex
from hurry.filesize import size

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from progressbar import AnimatedMarker, Bar, BouncingBar, Counter, ETA, \
    FileTransferSpeed, FormatLabel, Percentage, \
    ProgressBar, ReverseBar, RotatingMarker, \
    SimpleProgress, Timer, UnknownLength
from ussclicore.utils import generics_utils, printer, progressbar_widget, download_utils
from hammr.utils import *
from hammr.utils.hammr_utils import *
from hammr.utils.deployment_utils import *
from uforge.objects.uforge import *

#This import and configuration avoid pyxb warnings about xmls
import logging
logging.basicConfig()
logging.getLogger("pyxb").setLevel(logging.ERROR)

class Image(Cmd, CoreGlobal):
    """List, download or delete existing machine images. Publish new machine image to cloud account from configuration file"""

    cmd_name = "image"
    pbar = None

    def __init__(self):
        super(Image, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                  description="Displays all the machine images built and publish information of those machine images to their respective target platforms")
        return doParser

    def do_list(self, args):
        try:
            # call UForge API
            # get images
            printer.out("Getting all images and publications for [" + self.login + "] ...")
            images = self.api.Users(self.login).Images.Getall()
            images = images.images.image
            # get publications
            pimages = self.api.Users(self.login).Pimages.Getall()
            pimages = pimages.publishImages.publishImage
            if images is None or len(images) == 0:
                printer.out("No images available")
            else:
                printer.out("Images:")
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t", "t", "t", "t", "c", "t"])
                table.header(
                    ["Id", "Name", "Version", "Rev.", "Format", "Created", "Size", "Compressed", "Generation Status"])
                images = generics_utils.order_list_object_by(images, "name")
                for image in images:
                    imgStatus = self.get_image_status(image.status)
                    table.add_row([image.dbId, image.name, image.version, image.revision, image.targetFormat.name,
                                   image.created.strftime("%Y-%m-%d %H:%M:%S"), size(image.fileSize),
                                   "X" if image.compress else "", imgStatus])
                print table.draw() + "\n"
                printer.out("Found " + str(len(images)) + " images")

            if pimages is None or len(pimages) == 0:
                printer.out("No publication available")
            else:
                printer.out("Publications:")
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t", "t", "t", "t"])
                table.header(["Template name", "Image ID", "Publish ID", "Account name", "Format", "Cloud ID", "Status"])
                pimages = generics_utils.order_list_object_by(pimages, "name")
                for pimage in pimages:
                    pubStatus = self.get_publish_status(pimage.status)
                    table.add_row([pimage.name,
                                   generics_utils.extract_id(pimage.imageUri),
                                   pimage.dbId,
                                   pimage.credAccount.name if pimage.credAccount is not None else "-",
                                   pimage.credAccount.targetPlatform.name,
                                   pimage.cloudId if pimage.cloudId is not None else "-", pubStatus])
                print table.draw() + "\n"
                printer.out("Found " + str(len(pimages)) + " publications")

            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()

    def arg_publish(self):
        doParser = ArgumentParser(prog=self.cmd_name + " publish", add_help=True,
                                  description="Publish (upload and register) a built machine image to a target environment")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True,
                               help="yaml/json file providing the cloud account parameters required for upload and registration")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--id', dest='id', required=False, help="id of the image to publish")
        return doParser

    def do_publish(self, args):
        try:
            # add arguments
            doParser = self.arg_publish()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2
            template = validate(file)
            if template is None:
                return

            try:
                if doArgs.id:
                    images = self.api.Users(self.login).Images.Getall()
                    images = images.images.image
                    if images is None or len(images) == 0:
                        printer.out("No images available")
                    else:
                        for iimage in images:
                            if str(iimage.dbId) == str(doArgs.id):
                                image = iimage
                    if image is None:
                        printer.out("image not found", printer.ERROR)
                        return 2
                    if not self.is_image_ready_to_publish(image, None):
                        printer.out("Image with name '" + image.name + " can not be published", printer.ERROR)
                        return 2
                    appliance = self.api.Users(self.login).Appliances(
                        generics_utils.extract_id(image.applianceUri)).Get()
                    if appliance is None or not hasattr(appliance, 'dbId'):
                        printer.out("No template found for image", printer.ERROR)
                        return
                    rInstallProfile = self.api.Users(self.login).Appliances(appliance.dbId).Installprofile("").Get()
                    if rInstallProfile is None:
                        printer.out("No installation found on the template '" + template["stack"]["name"] + "'",
                                    printer.ERROR)
                        return
                    builder = self.find_builder(image, template)
                    if builder is None:
                        # TODO unmap image format
                        printer.out("No builder part found for image with format type: " + str(template["type"]),
                                    printer.ERROR)
                        return 2
                    self.publish_builder(builder, template, appliance, rInstallProfile, 1, image)
                else:
                    # Get template which correpond to the template file
                    appliances = self.api.Users(self.login).Appliances().Getall(
                        Query="name=='" + template["stack"]["name"] + "';version=='" + template["stack"][
                            "version"] + "'")
                    appliance = appliances.appliances.appliance
                    if appliance is None or len(appliance) != 1:
                        printer.out("No template found on the plateform", printer.ERROR)
                        return 0
                    appliance = appliance[0]
                    rInstallProfile = self.api.Users(self.login).Appliances(appliance.dbId).Installprofile("").Get()
                    if rInstallProfile is None:
                        printer.out("No installation found on the template '" + template["stack"]["name"] + "'",
                                    printer.ERROR)
                        return

                    i = 1
                    for builder in template["builders"]:
                        rCode = self.publish_builder(builder, template, appliance, rInstallProfile, i, None)
                        if rCode >= 2:
                            return
                        i += 1

            except KeyError as e:
                printer.out("unknown error template file, key: " + str(e), printer.ERROR)

        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_publish()
        except KeyboardInterrupt:
            pass
        except Exception as e:
            return handle_uforge_exception(e)

    def help_publish(self):
        doParser = self.arg_publish()
        doParser.print_help()

    def arg_deploy(self):
        doParser = ArgumentParser(prog=self.cmd_name + " deploy", add_help=True,
                                  description="Deploy an instance of a published image on the targeted cloud.")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True,
                               help="yaml/json file providing the instance parameters required for deployment on targeted cloud")
        mandatory.add_argument('--publish-id', dest='pid', required=True,
                               help="the ID of the published image to deploy")
        return doParser

    def do_deploy(self, args):
        try:
            # add arguments
            doParser = self.arg_deploy()
            doArgs = doParser.parse_args(shlex.split(args))

            # if the help command is called, parse_args returns None object
            if not doArgs:
                return 2

            pimage = self.get_pimage_from_id(doArgs.pid)
            if pimage == 2:
                return 2

            target_platform = ""
            if pimage.targetFormat:
                target_platform = pimage.targetFormat.name

            if not self.is_pimage_ready_to_deploy(pimage):
                printer.out("Published image with name '" + pimage.name + " cannot be deployed", printer.ERROR)
                return 2

            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2

            image_id = generics_utils.extract_id(pimage.imageUri)
            if image_id is None or image_id == "":
                printer.out("Image not found", printer.ERROR)
                return 2

            if "Amazon" in target_platform:
                return self.deploy_aws(file, pimage)

            if "OpenStack" in target_platform:
                return self.deploy_openstack(file, pimage)

            if "Azure" in target_platform:
                return self.deploy_azure(file, pimage)

            printer.out("Hammr only supports deployments for Amazon AWS, OpenStack and Microsoft Azure ARM.", printer.ERROR)

            return 2

        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_deploy()
        except KeyboardInterrupt:
            printer.out("You have exited the command-line, however the deployment may still be in progress. Please go to the cloud's console for more information", printer.WARNING)
            pass
        except Exception as e:
            return handle_uforge_exception(e)

    def help_deploy(self):
        doParser = self.arg_deploy()
        doParser.print_help()

    def arg_delete(self):
        doParser = ArgumentParser(prog=self.cmd_name + " delete", add_help=True,
                                  description="Deletes a machine image or publish information")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the machine image to delete")
        return doParser

    def do_delete(self, args):
        try:
            # add arguments
            doParser = self.arg_delete()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
            except SystemExit as e:
                return
            # call UForge API
            printer.out("Searching image with id [" + doArgs.id + "] ...")
            images = self.api.Users(self.login).Images.Getall()
            images = images.images.image
            if images is None or len(images) == 0:
                printer.out("No images available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t", "t", "t", "t", "c", "t"])
                table.header(["Id", "Name", "Version", "Rev.", "Format", "Created", "Size", "Compressed", "Status"])
                deleteImage = None
                for image in images:
                    if str(image.dbId) == str(doArgs.id):
                        imgStatus = self.get_image_status(image.status)
                        table.add_row([image.dbId, image.name, image.version, image.revision, image.targetFormat.name,
                                       image.created.strftime("%Y-%m-%d %H:%M:%S"), size(image.size),
                                       "X" if image.compress else "", imgStatus])
                        deleteImage = image
                if deleteImage is not None:
                    print table.draw() + "\n"
                    if generics_utils.query_yes_no(
                                    "Do you really want to delete image with id " + str(deleteImage.dbId)):
                        self.api.Users(self.login).Appliances(
                            generics_utils.extract_id(deleteImage.applianceUri)).Images(deleteImage.dbId).Delete()
                        printer.out("Image deleted", printer.OK)
                else:
                    printer.out("Image not found", printer.ERROR)


        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_delete(self):
        doParser = self.arg_delete()
        doParser.print_help()

    def arg_cancel(self):
        doParser = ArgumentParser(prog=self.cmd_name + " cancel", add_help=True,
                                  description="Cancels a machine image build or publish")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the machine image to cancel")
        return doParser

    def do_cancel(self, args):
        try:
            # add arguments
            doParser = self.arg_cancel()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
            except SystemExit as e:
                return
            # call UForge API
            printer.out("Searching image with id [" + doArgs.id + "] ...")
            images = self.api.Users(self.login).Images.Getall()
            images = images.images.image
            if images is None or len(images) == 0:
                printer.out("No images available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t", "t", "t", "t", "c", "t"])
                table.header(["Id", "Name", "Version", "Rev.", "Format", "Created", "Size", "Compressed", "Status"])
                cancelImage = None
                for image in images:
                    if str(image.dbId) == str(doArgs.id):
                        imgStatus = self.get_image_status(image.status)
                        table.add_row([image.dbId, image.name, image.version, image.revision, image.targetFormat.name,
                                       image.created.strftime("%Y-%m-%d %H:%M:%S"), size(image.size),
                                       "X" if image.compress else "", imgStatus])
                        print table.draw() + "\n"
                        cancelImage = image
                if cancelImage is None or cancelImage.status.complete or cancelImage.status.cancelled:
                    printer.out("Image not being generated, impossible to canceled", printer.ERROR)
                    return

                if cancelImage is not None:
                    if generics_utils.query_yes_no(
                                    "Do you really want to cancel image with id " + str(cancelImage.dbId)):
                        self.api.Users(self.login).Appliances(
                            generics_utils.extract_id(cancelImage.applianceUri)).Images(
                            cancelImage.dbId).Status.Cancel()
                        printer.out("Image Canceled", printer.OK)
                else:
                    printer.out("Image not found", printer.ERROR)


        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_cancel(self):
        doParser = self.arg_cancel()
        doParser.print_help()

    def arg_download(self):
        doParser = ArgumentParser(prog=self.cmd_name + " download", add_help=True,
                                  description="Downloads a machine image to the local filesystem")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the machine image to download")
        mandatory.add_argument('--file', dest='file', required=True,
                               help="the pathname where to store the machine image")
        return doParser

    def do_download(self, args):
        try:
            # add arguments
            doParser = self.arg_download()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
            except SystemExit as e:
                return
            # call UForge API
            printer.out("Searching image with id [" + doArgs.id + "] ...")
            images = self.api.Users(self.login).Images.Getall()
            images = images.images.image
            if images is None or len(images) == 0:
                printer.out("No images available")
            else:
                dlImage = None
                for image in images:
                    if str(image.dbId) == str(doArgs.id):
                        dlImage = image
                if dlImage is not None and dlImage.status.complete and not dlImage.status.error and dlImage.compress:
                    download_url = self.api.getUrl() + "/" + dlImage.downloadUri
                    dlUtils = download_utils.Download(download_url, doArgs.file, not self.api.getDisableSslCertificateValidation())
                    try:
                        dlUtils.start()
                    except Exception, e:
                        return
                    printer.out("Image downloaded", printer.OK)
                elif dlImage is None:
                    printer.out("Unable to find the image to download in your library", printer.ERROR)
                elif not dlImage.status.complete:
                    printer.out("The image is being generated. Unable to download. Please retry later", printer.ERROR)
                elif not dlImage.compress:
                    printer.out("The image has been prepared to be published (not compressed). Cannot download.",
                                printer.ERROR)
                else:
                    printer.out("Cannot download this image", printer.ERROR)


        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_download()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_download(self):
        doParser = self.arg_download()
        doParser.print_help()

    def get_image_status(self, status):
        if (status.complete and not status.error):
            imgStatus = "Done"
        elif status.error:
            imgStatus = "Error"
        elif status.cancelled:
            imgStatus = "Canceled"
        else:
            imgStatus = "In progress (" + str(status.percentage) + "%)"
        return imgStatus

    def get_publish_status(self, status):
        if (status.complete and not status.error):
            pubStatus = "Done"
        elif status.error:
            pubStatus = "Error"
        elif status.cancelled:
            pubStatus = "Canceled"
        else:
            pubStatus = "In progress (" + str(status.percentage) + "%)"
        return pubStatus

    def publish_builder(self, builder, template, appliance, rInstallProfile, i, comliantImage):
        try:
            if comliantImage is None:
                comliantImage = self.get_image_to_publish(builder, template, appliance, i)

            # get target format to define publish method
            format_type = comliantImage.targetFormat.format.name
            publishMethod = getattr(publish_utils, "publish_" + generics_utils.remove_special_chars(format_type), None)
            if publishMethod:
                mypImage = publishMethod(builder)
                if mypImage is None:
                    return 2
            else:
                printer.out("Builder type unknown: " + format_type, printer.ERROR)
                return 2

            mypImage.imageUri = comliantImage.uri
            mypImage.applianceUri = appliance.uri
            mypImage.credAccount = self.get_account_to_publish(builder)
            account_name = mypImage.credAccount.name

            rpImage = self.api.Users(self.login).Appliances(appliance.dbId).Images(
                comliantImage.dbId).Pimages().Publish(body=mypImage, element_name="ns1:publishImage")

            status = rpImage.status
            statusWidget = progressbar_widget.Status()
            statusWidget.status = status
            widgets = [Bar('>'), ' ', statusWidget, ' ', ReverseBar('<')]
            progress = ProgressBar(widgets=widgets, maxval=100).start()
            while not (status.complete or status.error or status.cancelled):
                statusWidget.status = status
                progress.update(status.percentage)
                status = self.api.Users(self.login).Appliances(appliance.dbId).Images(comliantImage.dbId).Pimages(
                    rpImage.dbId).Status.Get()
                time.sleep(2)
            statusWidget.status = status
            progress.finish()
            if status.error:
                printer.out("Publication to '" + builder["account"][
                    "name"] + "' error: " + status.message + "\n" + status.errorMessage, printer.ERROR)
                if status.detailedError:
                    printer.out(status.detailedErrorMsg)
            elif status.cancelled:
                printer.out("\nPublication to '" + builder["account"][
                    "name"] + "' canceled: " + status.message.printer.WARNING)
            else:
                printer.out("Publication to " + account_name + " is ok", printer.OK)
                rpImage = self.api.Users(self.login).Appliances(appliance.dbId).Images(comliantImage.dbId).Pimages(
                    rpImage.dbId).Get()
                if rpImage.cloudId is not None and rpImage.cloudId != "":
                    printer.out("Cloud ID : " + rpImage.cloudId)
            return 0
        except KeyboardInterrupt:
            printer.out("\n")
            if generics_utils.query_yes_no("Do you want to cancel the job ?"):
                if 'appliance' in locals() and 'comliantImage' in locals() and 'rpImage' in locals() \
                        and hasattr(appliance, 'dbId') and hasattr(comliantImage, 'dbId') and hasattr(rpImage, 'dbId'):
                    self.api.Users(self.login).Appliances(appliance.dbId).Images(comliantImage.dbId).Pimages(
                        rpImage.dbId).Cancel.Cancel()
                else:
                    printer.out("Impossible to cancel", printer.WARNING)
            else:
                printer.out("Exiting command")
            raise KeyboardInterrupt

    def is_image_ready_to_publish(self, image, builder):
        if builder is not None:
            if ("hardwareSettings" in builder) and ("memory" in builder["hardwareSettings"]) and (
                    not image.installProfile.memorySize == builder["hardwareSettings"]["memory"]):
                return False

            if ("installation" in builder) and ("swapSize" in builder["installation"]) and (
                    not image.installProfile.swapSize == builder["installation"]["swapSize"]):
                return False
                # print str(image.installProfile.swapSize)+"----"+ str(builder["installation"]["swapSize"])
                # TODO
                # if ("diskSize" in builder["installation"]) and (not image.installProfile.diskSize == builder["installation"]["diskSize"]):
                #        isOk = False
                #        print str(image.installProfile.diskSize)+"----"+ str(builder["installation"]["diskSize"])

        if not image.status.complete or image.status.error or image.status.cancelled:
            return False

        return True

    def is_pimage_ready_to_deploy(self, pimage):
        if not pimage.status.complete or pimage.status.error or pimage.status.cancelled:
            return False

        return True


    def find_builder(self, image, template):
        for builder in template["builders"]:
            if image.targetFormat.name == builder["type"]:
                return builder

        return None

    def get_image_to_publish(self, builder, template, appliance, i):
        printer.out(
            "Publishing '" + builder["type"] + "' image (" + str(i) + "/" + str(len(template["builders"])) + ")")
        images = self.api.Users(self.login).Appliances(appliance.dbId).Images.Getall(
            Query="targetFormat.name=='" + builder["type"] + "'")
        images = images.images.image
        if images is None or len(images) == 0:
            printer.out(
                "No images found for the template '" + template["stack"]["name"] + "' with type: " + builder[
                    "type"], printer.ERROR)
            return 2

        compliantImages = []
        for image in images:
            isOk = self.is_image_ready_to_publish(image, builder)
            if isOk:
                compliantImages.append(image)

        if (len(compliantImages) == 0):
            printer.out(
                "No images found for the template '" + template["stack"]["name"] + "' with type: " + builder[
                    "type"], printer.ERROR)
            return 2
        elif (len(compliantImages) == 1):
            comliantImage = compliantImages[0]
        else:
            # TODO get the last created image
            comliantImage = compliantImages[0]

        return comliantImage

    def get_account_to_publish(self, builder):
        if not "account" in builder:
            printer.out("Missing account section on builder: [" + builder["type"] + "]", printer.ERROR)
            return 2
        # Get all cloud account on the plateform (for the user)
        accounts = self.api.Users(self.login).Accounts.Getall()
        accounts = accounts.credAccounts.credAccount
        if accounts is None or len(accounts) == 0:
            printer.out("No accounts available on the plateform.\n You can use the command 'hammr account create' to create an account.", printer.ERROR)
            return 2
        else:
            for account in accounts:

                builder_name = self.get_account_name_for_publish(builder, account)
                if builder_name == "":
                    printer.out("No account name given", printer.ERROR)
                    return 2

                if account.name == builder_name:
                    # A hack to avoid a toDOM, toXML bug
                    account.targetPlatform.type._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'string')
                    if hasattr(account, 'certificates'):
                        account.certificates._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Certificates')
                        for c in account.certificates.certificate:
                            c.type._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'string')

                    return account
                    break

            printer.out("No accounts available with name " + builder["account"]["name"] + ".\nYou can use the command 'hammr account create' to create an account.", printer.ERROR)
            return 2

    # get the account name from field or from the credential file given in the builder
    def get_account_name_for_publish(self, builder, account):
        builder_name = ""
        if builder["account"].has_key("name"):
            builder_name = builder["account"]["name"]

        elif builder["account"].has_key("file"):

            file = generics_utils.get_file(builder["account"]["file"])
            if file is None:
                return ""
            template = validate(file)
            if template is None:
                return ""

            builder_name = self.get_account_name_from_template(template, builder)

        return builder_name

    def get_account_name_from_template(self, template, builder):
        account_name = ""
        if not template.has_key("accounts"):
            return ""

        for account in template["accounts"]:
            if account.has_key("type") and account["type"] == builder["type"] and account.has_key("name"):
                account_name = account["name"]
        return account_name

    def get_pimage_from_id(self, id):
        pimages = self.api.Users(self.login).Pimages.Getall()
        pimages = pimages.publishImages.publishImage
        pimage = None
        if pimages is None or len(pimages) == 0:
            printer.out("No published images available")
        else:
            for piimage in pimages:
                if str(piimage.dbId) == str(id):
                    pimage = piimage
        if pimage is None:
            printer.out("published image not found", printer.ERROR)
            return 2
        return pimage

    def deploy_aws(self, file, pimage):
        image_id = generics_utils.extract_id(pimage.imageUri)
        deployment = build_deployment_amazon(file)
        if deployment is None:
            return

        deployed_instance = call_deploy(self, pimage, deployment, image_id)
        if deployed_instance == 2:
            return 2
        deployed_instance_id = deployed_instance.applicationId

        print("Deployment in progress")

        status = show_deploy_progress_without_percentage(self, deployed_instance_id)
        return print_deploy_info(self, status, deployed_instance_id)

    def deploy_openstack(self, file, pimage):
        image_id = generics_utils.extract_id(pimage.imageUri)
        pid = pimage.dbId
        bar_status = OpStatus()
        progress = create_progress_bar_openstack(bar_status)
        deployment = build_deployment_openstack(file, pimage, pid, retrieve_credaccount(self, pid, pimage))
        if deployment is None:
            return

        bar_status.percentage = 50
        bar_status.message = "Deploying instance"
        progress.update(bar_status.percentage)

        deployed_instance = call_deploy(self, pimage, deployment, image_id)
        if deployed_instance == 2:
            return 2
        deployed_instance_id = deployed_instance.applicationId

        status = show_deploy_progress_with_percentage(self, deployed_instance_id, bar_status, progress)
        return print_deploy_info(self, status, deployed_instance_id)

    def deploy_azure(self, file, pimage):
        image_id = generics_utils.extract_id(pimage.imageUri)
        deployment = build_deployment_azure(file)
        if deployment is None:
            return

        deployed_instance = call_deploy(self, pimage, deployment, image_id)
        if deployed_instance == 2:
            return 2
        deployed_instance_id = deployed_instance.applicationId

        print("Deployment in progress")

        status = show_deploy_progress_without_percentage(self, deployed_instance_id)
        return print_deploy_info(self, status, deployed_instance_id)


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ussclicore.utils import printer
from uforge.objects.uforge import *


def publish_vcd(pimage, builder):
    # doing field verification
    if not "orgName" in builder:
        printer.out("orgName in vcd builder not found", printer.ERROR)
        return
    if not "catalogName" in builder:
        printer.out("catalogName in vcd builder not found", printer.ERROR)
        return
    if not "imageName" in builder:
        printer.out("imageName in vcd builder not found", printer.ERROR)
        return

    pimage.credAccount.organizationName = builder["orgName"]
    pimage.credAccount.catalogId = builder["catalogName"]
    pimage.credAccount.displayName = builder["imageName"]
    return pimage


def publish_vcenter(builder):
    pimage = PublishImageVSphere()

    # doing field verification

    if not "displayName" in builder:
        printer.out("displayName in vcenter builder not found", printer.ERROR)
        return

    if not "esxHost" in builder:
        printer.out("esxHost in vcenter builder not found", printer.ERROR)
        return

    if not "datastore" in builder:
        printer.out("datastore in vcenter builder not found", printer.ERROR)
        return

    if "network" in builder:
        pimage.network = builder["network"]

    pimage.displayName = builder["displayName"]
    pimage.esxHost = builder["esxHost"]
    pimage.datastore = builder["datastore"]
    return pimage


def publish_cloudstack(pimage, builder):
    # doing field verification
    if not "imageName" in builder:
        printer.out("imageName in cloudstack builder not found", printer.ERROR)
        return
    if not "zone" in builder:
        printer.out("zone in cloudstack builder not found", printer.ERROR)
        return
    if "publicImage" in builder:
        pimage.credAccount.publicImage = True if (builder["publicImage"] == "true") else False
    if "featured" in builder:
        pimage.credAccount.featuredEnabled = True if (builder["featured"] == "true") else False

    pimage.credAccount.displayName = builder["imageName"]
    pimage.credAccount.zoneName = builder["zone"]
    return pimage


def publish_cloudstack_qcow2(pimage, builder):
    return publish_cloudstack(pimage, builder)


def publish_cloudstack_vhd(pimage, builder):
    return publish_cloudstack(pimage, builder)


def publish_cloudstack_ova(pimage, builder):
    return publish_cloudstack(pimage, builder)


def publish_susecloud(pimage, builder):
    # doing field verification
    if not "imageName" in builder:
        printer.out("imageName in susecloud builder not found", printer.ERROR)
        return
    if not "tenant" in builder:
        printer.out("tenant in susecloud builder not found", printer.ERROR)
        return
    if "description" in builder:
        pimage.credAccount.description = builder["description"]

    pimage.credAccount.displayName = builder["imageName"]
    pimage.credAccount.tenantName = builder["tenant"]
    if "publicImage" in builder:
        pimage.credAccount.publicImage = True if (builder["publicImage"] == "true") else False
    # if "paraVirtualMode" in builder:
    #        pimage.credAccount. = True if (builder["paraVirtualMode"]=="true") else False
    return pimage


def publish_openstack(builder):
    pimage = PublishImageOpenStack()

    # doing field verification
    if not "displayName" in builder:
        printer.out("displayName in openstack builder not found", printer.ERROR)
        return
    if not "tenantName" in builder:
        printer.out("TenantName in openstack builder not found", printer.ERROR)
        return

    pimage.displayName = builder["displayName"]
    pimage.tenantName = builder["tenantName"]

    if "publicImage" in builder:
        pimage.credAccount.publicImage = True if (builder["publicImage"] == "true") else False
    if "keystoneDomain" in builder:
        pimage.keystoneDomain = builder["keystoneDomain"]
        return
    if "keystoneProject" in builder:
        pimage.keystoneProject = builder["keystoneProject"]
        return
    return pimage


def publish_openstackqcow2(builder):
    return publish_openstack(builder)


def publish_openstackvhd(pimage, builder):
    return publish_openstack(pimage, builder)


def publish_openstackvmdk(pimage, builder):
    return publish_openstack(pimage, builder)


def publish_openstackvdi(pimage, builder):
    return publish_openstack(pimage, builder)


def publish_aws(builder):
    pimage = PublishImageAws()

    # doing field verification
    if not "bucket" in builder:
        printer.out("bucket in AWS builder not found", printer.ERROR)
        return
    if not "region" in builder:
        printer.out("region in AMI builder not found", printer.ERROR)
        return

    pimage.bucket = builder["bucket"]
    pimage.region = builder["region"]
    return pimage


def publish_azure(builder):
    if "blob" in builder or "container" in builder:
        printer.out("Azure Resource Manager publish")
        return publish_azure_arm(builder)
    else:
        printer.out("Azure classic publish")
        return publish_azure_classic(builder)


def publish_azure_classic(builder):
    pimage = PublishImageAzure()

    # doing field verification
    if not "storageAccount" in builder:
        printer.out("storageAccount in Microsoft Azure not found", printer.ERROR)
        return
    if not "region" in builder:
        printer.out("region in Microsoft Azure not found", printer.ERROR)
        return

    pimage.storageAccount = builder["storageAccount"]
    pimage.region = builder["region"]

    return pimage


def publish_azure_arm(builder):
    pimage = PublishImageAzureResourceManager()

    if not "storageAccount" in builder:
        printer.out("storageAccount not found", printer.ERROR)
        return
    if not "container" in builder:
        printer.out("container not found", printer.ERROR)
        return
    if not "blob" in builder:
        printer.out("blob not found", printer.ERROR)
        return
    if not "displayName" in builder:
        printer.out("displayName not found", printer.ERROR)
        return

    if "resourceGroup" in builder:
        pimage.resourceGroup = builder["resourceGroup"]

    pimage.storageAccount = builder["storageAccount"]
    pimage.container = builder["container"]
    pimage.blob = builder["blob"]
    pimage.displayName = builder["displayName"]

    return pimage


def publish_flexiant(builder):
    pimage = PublishImageFlexiant()

    # doing field verification
    if not "diskOffering" in builder:
        printer.out("diskOffering in flexiant builder not found", printer.ERROR)
        return
    if not "virtualDatacenterName" in builder:
        printer.out("virtualDatacenterName in flexiant builder not found", printer.ERROR)
        return
    if not "machineImageName" in builder:
        printer.out("machineImageName in flexiant builder not found", printer.ERROR)
        return

    pimage.diskOffering = builder["diskOffering"]
    pimage.virtualDatacenterName = builder["virtualDatacenterName"]
    pimage.machineImageName = builder["machineImageName"]

    return pimage


def publish_flexiant_kvm(pimage, builder):
    return publish_flexiant(pimage, builder)


def publish_flexiant_ova(pimage, builder):
    return publish_flexiant(pimage, builder)


def publish_flexiantraw(builder):
    return publish_flexiant(builder)


def publish_abiquo(pimage, builder):
    # doing field verification
    if not "enterprise" in builder:
        printer.out("enterprise in abiquo builder not found", printer.ERROR)
        return
    if not "datacenter" in builder:
        printer.out("datacenter in abiquo builder not found", printer.ERROR)
        return
    if not "productName" in builder:
        printer.out("productName in abiquo builder not found", printer.ERROR)
        return
    if not "category" in builder:
        printer.out("category in abiquo builder not found", printer.ERROR)
        return
    if not "description" in builder:
        printer.out("description in abiquo builder not found", printer.ERROR)
        return

    pimage.credAccount.datacenterName = builder["datacenter"]
    pimage.credAccount.displayName = builder["productName"]
    pimage.credAccount.category = builder["category"]
    pimage.credAccount.organizationName = builder["enterprise"]
    pimage.credAccount.description = builder["description"]

    return pimage


def publish_nimbula(pimage, builder):
    # doing field verification
    if not "imageListName" in builder:
        printer.out("imageListName in nimbula builder not found", printer.ERROR)
        return
    if not "imageVersion" in builder:
        printer.out("imageVersion in nimbula builder not found", printer.ERROR)
        return
    if not "description" in builder:
        printer.out("description in nimbula builder not found", printer.ERROR)
        return

    pimage.credAccount.imageVersion = builder["imageVersion"]
    pimage.credAccount.description = builder["description"]
    pimage.credAccount.listName = builder["imageListName"]

    return pimage


def publish_nimbula_kvm(pimage, builder):
    return publish_nimbula(pimage, builder)


def publish_nimbula_esx(pimage, builder):
    return publish_nimbula(pimage, builder)


def publish_eucalyptus(pimage, builder):
    # doing field verification
    if not "imageName" in builder:
        printer.out("imageName in Eucalyptus builder not found", printer.ERROR)
        return
    if not "description" in builder:
        printer.out("description in Eucalyptus builder not found", printer.ERROR)
        return
    if not "bucket" in builder:
        printer.out("bucket in Eucalyptus builder not found", printer.ERROR)
        return

    pimage.credAccount.displayName = builder["imageName"]
    pimage.credAccount.bucket = builder["bucket"]
    pimage.credAccount.description = builder["description"]
    if "ramdisk" in builder and "kernelId" in builder:
        pimage.credAccount.ramdiskId = builder["ramdisk"]
        pimage.credAccount.kernelId = builder["kernelId"]

    return pimage


def publish_eucalyptus_kvm(pimage, builder):
    return publish_eucalyptus(pimage, builder)


def publish_eucalyptus_xen(pimage, builder):
    return publish_eucalyptus(pimage, builder)


def publish_gce(pimage, builder):
    # doing field verification
    if not "computeZone" in builder:
        printer.out("computeZone in GCE builder not found", printer.ERROR)
        return
    if not "bucketLocation" in builder:
        printer.out("bucketLocation in GCE builder not found", printer.ERROR)
        return
    if not "bucket" in builder:
        printer.out("bucket in GCE builder not found", printer.ERROR)
        return
    if not "projectId" in builder:
        printer.out("projectId in GCE builder not found", printer.ERROR)
        return
    if not "storageClass" in builder:
        printer.out("storageClass in GCE builder not found", printer.ERROR)
        return
    if not "diskNamePrefix" in builder:
        printer.out("diskNamePrefix in AMI builder not found", printer.ERROR)
        return

    if "description" in builder:
        pimage.credAccount.description = builder["description"]

    pimage.credAccount.bucket = builder["bucket"]
    pimage.credAccount.tenantName = builder["projectId"]
    pimage.credAccount.category = builder["storageClass"]
    pimage.credAccount.displayName = builder["diskNamePrefix"]
    pimage.credAccount.zoneName = builder["computeZone"]
    pimage.publishLocation = builder["bucketLocation"]
    return pimage


def publish_outscale(pimage, builder):
    # doing field verification
    if not "zone" in builder:
        printer.out("zone in outscale builder not found", printer.ERROR)
        return
    if not "description" in builder:
        pimage.credAccount.description = builder["description"]

    pimage.credAccount.zoneName = builder["zone"]
    return pimage


def publish_k5vmdk(builder):
    pimage = PublishImageK5()

    # doing field verification
    if not "displayName" in builder:
        printer.out("displayName in k5 builder not found", printer.ERROR)
        return
    if not "domain" in builder:
        printer.out("domain in k5 builder not found", printer.ERROR)
        return
    if not "project" in builder:
        printer.out("project in k5 builder not found", printer.ERROR)
        return
    if not "region" in builder:
        printer.out("region in k5 builder not found", printer.ERROR)
        return

    pimage.displayName = builder["displayName"]
    pimage.keystoneDomain = builder["domain"]
    pimage.keystoneProject = builder["project"]
    pimage.publishLocation = builder["region"]

    return pimage


def publish_docker(builder):
    pimage = PublishImageDocker()

    if not "namespace" in builder:
        printer.out("namespace in Docker builder is missing", printer.ERROR)
        return
    if not "repositoryName" in builder:
        printer.out("repositoryName in Docker builder is missing", printer.ERROR)
        return
    if not "tagName" in builder:
        printer.out("tagName in Docker builder is missing", printer.ERROR)
        return

    pimage.namespace = builder["namespace"]
    pimage.repositoryName = builder["repositoryName"]
    pimage.tagName = builder["tagName"]
    return pimage



# Copyright 2007-2017 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

from hammr.utils.generate_utils import *

class TestGenerateK5(TestCase):
    def test_generate_k5vmdk_should_return_uncompressed_image_given_compressed_image(self):
        # given
        image_given = CompressedImage()
        intall_profile_given = MockObject()

        # when
        image, install_profile = generate_k5vmdk(image_given, WhateverObject(), intall_profile_given, WhateverObject(), WhateverObject())

        # then
        self.assertFalse(image.compress)
        self.assertEquals(intall_profile_given, install_profile)


    def test_generate_k5vmdk_should_return_uncompressed_image_given_uncompressed_image(self):
        # given
        image_given = UncompressedImage()
        intall_profile_given = MockObject()

        # when
        image, install_profile = generate_k5vmdk(image_given, WhateverObject(), intall_profile_given,
                                                 WhateverObject(), WhateverObject())

        # then
        self.assertFalse(image.compress)
        self.assertEquals(intall_profile_given, install_profile)


class TestGeneratePXE(TestCase):
    def test_generate_pxe_should_return_uncompressed_image_given_compressed_image(self):
        # given
        image_given = CompressedImage()
        intall_profile_given = MockObject()

        # when
        image, install_profile = generate_pxe(image_given, WhateverObject(), intall_profile_given, None, None)

        # then
        self.assertFalse(image.compress)
        self.assertEquals(intall_profile_given, install_profile)


    def test_generate_pxe_should_return_uncompressed_image_given_uncompressed_image(self):
        # given
        image_given = UncompressedImage()
        intall_profile_given = MockObject()

        # when
        image, install_profile = generate_pxe(image_given, WhateverObject(), intall_profile_given,
                                                 None, None)

        # then
        self.assertFalse(image.compress)
        self.assertEquals(intall_profile_given, install_profile)

class CompressedImage:
    compress = True

class UncompressedImage:
    compress = False

class MockObject:
    first_attribute = "something"
    second_attribute = "something else"

class WhateverObject:
    whatever = "whatever"